/*********************************************** {COPYRIGHT-TOP} ***
* IBM Confidential
* OCO Source Materials
* 5724-V51
*
* (c) Copyright IBM Corp. 2000, 2008 All Rights Reserved.
*
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has been
* deposited with the U.S. Copyright Office.
************************************************ {COPYRIGHT-END} **/



/**
 *  Copyright IBM Corp. and other(s) 2000, 2008 All Rights Reserved. 
 *
 * The following sample of source code ("Sample Code") is owned by International Business Machines Corporation or 
 * one of its subsidiaries ("IBM") and is copyrighted and licensed, not sold.
 * You may use, copy, modify, and distribute the Sample Code in any form without payment to IBM, for the purpose of assisting you in the development of your applications.

 * The Sample Code is provided to you on an "AS IS" basis, without warranty of any kind. 
 * IBM HEREBY EXPRESSLY DISCLAIMS ALL WARRANTIES, EITHER EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 * Some jurisdictions do not allow for the exclusion or limitation of implied warranties, so the above limitations or exclusions may not apply to you. 
 * IBM shall not be liable for any damages you suffer as a result of using, copying, modifying or distributing the Sample Code, even if IBM has been advised of the possibility of such damages.
 **/
package com.ibm.ccd.loader;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;	
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Properties;

import com.ibm.pim.catalog.item.Item;
import com.ibm.pim.context.Context;
import com.ibm.pim.context.PIMContextFactory;
import com.ibm.pim.docstore.DocstoreManager;
import com.ibm.pim.docstore.Document;
import com.ibm.pim.utils.Logger;


public class ProcessorUtils 
{	
	private static Context context;
	private static Logger logger;
	private static final String DB_CLASSNAME = "db_class_name";
	private static final String DB_URL = "db_url";
	private static final String DB_USERNAME = "db_userName";
	private static final String DB_PASSWORD = "db_password";

	public static class DBProperties
	{		
		public String dbDriver;
		public String dbURL;
		public String dbUsername;
		public String dbPassword;
	}   

	/***
	 * This method is used to get either a FileReader or a Reader to get the content of 
	 * a document in the docstore. The path here could correspond either to a 
	 * filesystem path or to a docstore path
	 * @param path - path can be a docstore path or a fileSystem path
	 * @return Reader Object - returns the Reader corresponding to the docstore or the file or returns null when the docstore path is invalid
	 * @throws FileNotFoundException - When the file(data file or the property file) does not exist
	 * @throws Exception - when getting the context
	 * @throws PIMAuthorizationException - when user is not Aythorized to do the operation
	 */
	public static Reader getReader(String path) throws FileNotFoundException,Exception
	{
		Reader reader = null;
		String fileStartPath = "file://";  
		if(path.startsWith(fileStartPath))
		{   
			reader = new FileReader(path.substring(fileStartPath.length()));         	
		}
		else
		{         
			context = PIMContextFactory.getCurrentContext();
			DocstoreManager docstoreMgr = context.getDocstoreManager();
			Document doc = docstoreMgr.getDocument(path);
			if(doc != null)
			{
				reader = doc.getContentReader();
			}
            else
            {
                throw new IllegalArgumentException("Document at path " + path + " is not found in the docstore");
            }
		}	
		return reader;         
	}

	/***
	 * Reads the properties db_class_name,db_url,db_username,db_password set in the common.properties
	 * These properties are used to get the connection to the WPC Schema.
	 * @return The properties object
	 * @throws FileNotFoundException - when the common.properties is not found
	 * @throws IOException - occurs when parsing common.properties file
	 * @throws Exception - occurs while getting the context
	 */
	private static DBProperties readDatabaseProperties()throws FileNotFoundException,IOException,Exception
	{
		DBProperties dbProperties = new DBProperties();
		//get the service name service name 
		String svcName = System.getProperty("svc_name");
		//default service name
		String defaultSvcName = "default";

		String CCD_ETC_DIR = "CCD_ETC_DIR";
		String COMMON_PROPERTY_FILE = "/common.properties";

		context = PIMContextFactory.getCurrentContext();
		logger = context.getDefaultLogger();

		if(svcName == null)
		{
			svcName = defaultSvcName;
		}
		String properties_file = System.getProperty(CCD_ETC_DIR)+"/"+ svcName+COMMON_PROPERTY_FILE;
		if(!new File(properties_file).exists())
		{
			properties_file = System.getProperty(CCD_ETC_DIR)+"/"+ defaultSvcName+COMMON_PROPERTY_FILE;
		}   
		Properties p = new Properties();
		try 
		{	
			p.load(new FileInputStream(properties_file));
		}
		catch (FileNotFoundException e) 
		{
			logger.logError("Common.Properties file not identified");
			throw(e);
		}
		catch (IOException ioe) 
		{	
			logger.logError("IO Exception loading JDBC properties from Common.properties");
			throw(ioe);
		}    

		dbProperties.dbDriver = p.getProperty(DB_CLASSNAME);       
		dbProperties.dbURL = p.getProperty(DB_URL);
		dbProperties.dbUsername = p.getProperty(DB_USERNAME);
		dbProperties.dbPassword = p.getProperty(DB_PASSWORD);
		return dbProperties;	
	}

	/***
	 * 
	 * @param dbDriver - properties read from common.properties to get connection
	 * @param dbURL
	 * @param dbUsername 
	 * @param dbPassword 
	 * @return Connection object - returns the Connection Object
	 */
	public static Connection getConnection()throws ClassNotFoundException,SQLException,FileNotFoundException,IOException,Exception
	{
		DBProperties dbProperties = readDatabaseProperties();
		Connection conn = null;	           
		Class.forName(dbProperties.dbDriver); 			
		conn = DriverManager.getConnection(dbProperties.dbURL, dbProperties.dbUsername, dbProperties.dbPassword);            
		return conn;
	}

	/***
	 * 
	 * @param statusTableConnObj - connection to status table
	 * @param jobId 
	 * @param recordId
	 * @param status
	 * @param timestamp
	 * @throws SQLException - error when updating the status table
	 */
	public static void updateItemStatus(PreparedStatement stmt,                                        
                                        int jobId,
                                        int recordId,
                                        String status, 
                                        long timestamp) throws SQLException
	{ 
        stmt.setInt(1, jobId);
        stmt.setInt(2, recordId);
        stmt.setString(3, status);                
        stmt.setDate(4, new java.sql.Date(timestamp));
        		        
		stmt.execute();  	
	}

	/***
	 * 
	 * @param item - item to be checked out
	 * @param WorkflowIdentifierImpl is the implementation class of the WorkflowIdentifier interface
	 * @return the collaboration area name corresponding to the item
	 * @throws Exception - when getting the context
	 */
	public static String getCollaborationAreaforItem(Item item, String workflowIdentifierImpl)throws Exception
	{
		String loggerName ="JavaAPI_Exception";
		context = PIMContextFactory.getCurrentContext();
		logger = context.getLogger(loggerName);

		String collabArea = null;	   
		try
		{
			Class extnClass = Class.forName(workflowIdentifierImpl);
			WorkflowIdentifier wflIdentifier = 
				(WorkflowIdentifier)extnClass.newInstance();
			collabArea = wflIdentifier.getCollaborationAreaForItem(item);
		}
		catch( ClassNotFoundException e)
		{
			logger.logError(e);	
		}
		catch( IllegalAccessException e)
		{
			logger.logError(e);	
		}
		catch(InstantiationException e)	
		{
			logger.logError(e);
		}
		return collabArea;
	}
}     
